#include "unix_system.hpp"

std::string texmacs_utf8_string_to_system_string(string utf8_string) {
  return {&utf8_string[0], (std::basic_string<char>::size_type)N(utf8_string)};
}

string texmacs_system_string_to_utf8_string(const std::string &local8bit_string) {
  return {local8bit_string.data(), (int)local8bit_string.size()};
}

void texmacs_lock_file(FILE *&file) {
  int file_descriptor = fileno(file);
  if (flock(file_descriptor, LOCK_EX) == -1) {
    fclose(file);
    file = nullptr;
  }
}

void texmacs_unlock_file(FILE *&file) {
  int file_descriptor = fileno(file);
  flock(file_descriptor, LOCK_UN);
}


FILE* texmacs_fopen(string filename, string mode, bool lock) {
  c_string c_mode = mode;
  FILE *file = fopen(texmacs_utf8_string_to_system_string(filename).c_str(), c_mode);
  if (file == nullptr) {
    return nullptr;
  }
  if (lock) {
    texmacs_lock_file(file);
  }
  return file;
}


void texmacs_fclose(FILE *&file, bool unlock) {
  if (unlock) {
    texmacs_unlock_file(file);
  }
  fclose(file);
  file = nullptr;
}

TEXMACS_DIR texmacs_opendir(string dirname) {
  return (TEXMACS_DIR)opendir(texmacs_utf8_string_to_system_string(dirname).c_str());
}

void texmacs_closedir(TEXMACS_DIR dir) {
  closedir((DIR*)dir);
  dir = nullptr;
}

texmacs_dirent texmacs_readdir(TEXMACS_DIR dirp) {
  struct dirent* entry = readdir(dirp);
  if (entry == NULL) {
    return {false, ""};
  }
  return {true, texmacs_system_string_to_utf8_string(entry->d_name)};
}

bool texmacs_stat(string filename, struct_stat* buf) {
  return stat(texmacs_utf8_string_to_system_string(filename).c_str(), buf);
}

bool texmacs_mkdir(string dirname, int mode) {
  return mkdir(texmacs_utf8_string_to_system_string(dirname).c_str(), mode);
}

bool texmacs_rmdir(string dirname) {
  return rmdir(texmacs_utf8_string_to_system_string(dirname).c_str());
}

bool texmacs_rename(string oldname, string newname) {
  return rename(texmacs_utf8_string_to_system_string(oldname).c_str(), texmacs_utf8_string_to_system_string(newname).c_str());
}

bool texmacs_chmod(string filename, int mode) {
  return chmod(texmacs_utf8_string_to_system_string(filename).c_str(), mode);
}

bool texmacs_remove(string filename) {
  return remove(texmacs_utf8_string_to_system_string(filename).c_str()) != 0;
}

string texmacs_getenv(string variable_name) {
    c_string _variable_name = variable_name;
    char *value = getenv(_variable_name);
    if (value == nullptr) {
        return "";
    }
    return value;
}

bool texmacs_setenv(string variable_name, string new_value) {
    c_string _variable_name = variable_name;
    c_string _new_value = new_value;
    return setenv(_variable_name, _new_value, 1) == 0;
}

bool texmacs_putenv(string variable) {
    c_string _variable = variable;
    return putenv(_variable) == 0;
}


int texmacs_main(int argc, char** argv);

int main(int argc, char** argv) {
  return texmacs_main(argc, argv);
}
