#ifndef TEXMACS_UNIX_SYSTEM_HPP
#define TEXMACS_UNIX_SYSTEM_HPP

#include <sys/file.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include <string>

#include "string.hpp"
#include "unix_system.hpp"

typedef DIR* TEXMACS_DIR;
typedef struct stat struct_stat;

typedef struct texmacs_dirent {
    bool           is_valid;     /* entry is valid */
    string         d_name;       /* nom du fichier */
} texmacs_dirent;

void texmacs_lock_file(FILE *&file);
void texmacs_unlock_file(FILE *&file);
FILE* texmacs_fopen(string filename, string mode, bool lock = true);
void texmacs_fclose(FILE *&file, bool unlock = true);

TEXMACS_DIR texmacs_opendir(string dirname);
void texmacs_closedir(TEXMACS_DIR dir);
texmacs_dirent texmacs_readdir(TEXMACS_DIR dirp);

bool texmacs_stat(string filename, struct_stat* buf);
bool texmacs_mkdir(string dirname, int mode);
bool texmacs_rmdir(string dirname);
bool texmacs_rename(string oldname, string newname);
bool texmacs_chmod(string filename, int mode);
bool texmacs_remove(string filename);

string texmacs_getenv(string variable_name);
bool texmacs_setenv(string variable_name, string new_value);
bool texmacs_putenv(string variable);

#endif // TEXMACS_UNIX_SYSTEM_HPP